% For each Pi in the range [PiMin;PiMax]
% the function computes the GMPR and MPR interfaces for a given taskset.
% The preference is given to interfaces with minimum value of m.
% The input schedPolicy is a local scheduling policy for a task set, which
% is "GEDF" or "FP". If schedPolicy is not specified, then computations are
% performed for both GEDF and FP.
%
% The results of computations are displayed on plots as U(interface) =
% f(Pi);
function simulations_varing_Pi_and_m(tasks, PiMin, PiMax, mMax, schedPolicy)

% If no scheduling policy is selected,
% computations are performed for both GEDF and FP
if nargin < 5
    schedPolicy = '';
end

for Pi = PiMin:PiMax
    
    disp('Pi');
    disp(Pi);
    
    Pis(Pi-PiMin+1) = Pi;
    
    mMin = ceil(get_utilization(tasks));
    
    gmprInterfacesEDF = [];
    mprInterfacesEDF = [];
    gmprInterfacesFP = [];
    mprInterfacesFP = [];
    
    % Enumerate m until all interfaces are found.
    % The preference is given to the interfaces with minimal m.
    for m = mMin:mMax
        
        if (~strcmp(schedPolicy,'FP'))
            if (check_m_feasibility(m,Pi,tasks,'GEDF'))
                if (numel(gmprInterfacesEDF) == 0)
                    gmprInterfacesEDF = compute_gmpr(Pi, m, tasks, 'GEDF');
                end
                
                if (numel(mprInterfacesEDF) == 0)
                    mprInterfacesEDF = compute_mpr(Pi, m, tasks, 'GEDF');
                end
            end
        end
        
        if (~strcmp(schedPolicy,'GEDF'))
            if (check_m_feasibility(m,Pi,tasks,'FP'))
                if (numel(gmprInterfacesFP) == 0)
                    gmprInterfacesFP = compute_gmpr(Pi, m, tasks, 'FP');
                end
                
                if (numel(mprInterfacesFP) == 0)
                    mprInterfacesFP = compute_mpr(Pi, m, tasks, 'FP');
                end
            end
        end
        
        % Check if all interfaces have been generated for a given Pi
        if strcmp(schedPolicy,'GEDF')
            if ((numel(gmprInterfacesEDF) == 0) || (numel(mprInterfacesEDF) == 0))
                simulationInvalid = 1;
                break;
            end
        elseif strcmp(schedPolicy,'FP')
            if ((numel(gmprInterfacesFP) == 0) || (numel(mprInterfacesFP) == 0))
                simulationInvalid = 1;
                break;
            end
        elseif (numel(gmprInterfacesEDF) > 0) || ((numel(mprInterfacesEDF) > 0) || (numel(gmprInterfacesFP) > 0) || (numel(mprInterfacesFP) > 0))
            break;
        end
    end % end for m iteration
    
    
    if (~strcmp(schedPolicy,'FP') && numel(gmprInterfacesEDF) == 0)
        disp('No feasible GMPR interface for the specified hardware platform (EDF scheduler)');
        return;
    elseif (~strcmp(schedPolicy,'FP') && numel(mprInterfacesEDF) == 0)
        disp('No feasible MPR interface for the specified hardware platform (EDF scheduler)');
        return;
    elseif (~strcmp(schedPolicy,'GEDF') && numel(gmprInterfacesFP) == 0)
        disp('No feasible GMPR interface for the specified hardware platform (FP scheduler)');
        return;
    elseif (~strcmp(schedPolicy,'GEDF') && numel(mprInterfacesFP) == 0)
        disp('No feasible MPR interface for the specified hardware platform (FP scheduler)');
        return;
    end
    
   
    % Select optimal GMPR interfaces.
    % Optimal GMPR interface is the one with a minimal \Theta_m
    % GMPR EDF
    if (~strcmp(schedPolicy,'FP'))
        optimalGmprInterfacesEDF = get_optimal_gmpr_interfaces(gmprInterfacesEDF);
        gmprThetaEDF = optimalGmprInterfacesEDF(1).thetas(end);
        gmprU_EDF = gmprThetaEDF / Pi;
        GmprUs_EDF(Pi-PiMin+1) = gmprU_EDF;
        disp('GMPR U'); disp(gmprU_EDF);
        
        % MPR EDF
        mprThetaEDF = mprInterfacesEDF.theta;
        mprU_EDF = mprThetaEDF / Pi;
        MprUs_EDF(Pi-PiMin+1) = mprU_EDF;
        disp('MPR U'); disp(mprU_EDF);
    end
    
    % GMPR FP
    if (~strcmp(schedPolicy,'GEDF'))
        optimalGmprInterfacesFP = get_optimal_gmpr_interfaces(gmprInterfacesFP);
        gmprThetaFP = optimalGmprInterfacesFP(1).thetas(end);
        gmprU_FP = gmprThetaFP / Pi;
        GmprUs_FP(Pi-PiMin+1) = gmprU_FP;
        
        % MPR FP
        mprThetaFP = mprInterfacesFP.theta;
        mprU_FP = mprThetaFP / Pi;
        MprUs_FP(Pi-PiMin+1) = mprU_FP;
    end
end

% Plot the results
plot_interface_Us_for_Pis(Pis, GmprUs_EDF, MprUs_EDF, GmprUs_FP, MprUs_FP, 'U_{\{GMPR,MPR\}} = f(\Pi )', schedPolicy);

end